# File MD5

Module helps to uniquely identify the files using their MD5 hashes.

All that this module does - extends the `file_managed` table with `md5` column and, on file saving, set a value for it.

## MD5 validation

Module provides an implementation of `hook_file_validate()` to validate file duplicates. This validation could be disabled here: `admin/config/media/file-system`.

## Example implementation

- Create a form with submit callback.

```php
/**
 * Implements hook_form().
 */
function MODULE_form(array $form, array &$form_state) {
  $form['title'] = [
    '#type' => 'textfield',
    '#title' => t('Title'),
    '#default_value' => '',
  ];

  $form['file'] = [
    '#type' => 'managed_file',
    '#title' => t('File'),
    '#default_value' => 0,
  ];

  foreach ($form as $name => $item) {
    $value = variable_get($name, $item['#default_value']);

    // Preprocess value of file field.
    if (isset($item['#type']) && 'managed_file' === $item['#type']) {
      // 32 - is a length of MD5 hash of a file.
      // @see _file_md5_schema_file_managed_field_definitions()
      $file = strlen($value) === 32 ? file_md5_load_by('md5', $value) : file_load($value);

      if (FALSE !== $file) {
        $value = $file->fid;
      }
    }

    $form[$name]['#default_value'] = $value;
  }

  $form['#submit'][] = 'MODULE_form_submit';

  return system_settings_form($form);
}
```

- Save MD5 hash instead of file ID.

```php
/**
 * Implements hook_form_submit().
 */
function MODULE_form_submit(array $form, array &$form_state) {
  global $user;

  $file = file_load($form_state['values']['file']);
  $file->status = FILE_STATUS_PERMANENT;

  // Replace FID by MD5.
  $form_state['values']['file'] = $file->md5;

  // Add file usage.
  file_usage_add(file_save($file), 'MODULE', 'user', $user->uid);
}
```

- Export configuration via [Features](https://drupal.org/project/features).
- Add files to version control system.
- Reinstall the site and add files into database.

```php
/**
 * Implements hook_install().
 */
function MODULE_install() {
  global $user;

  foreach (file_md5_scan('public://', ['jpg', 'jpeg', 'csv', 'png', 'docx', 'pdf', 'gif', 'doc']) as $file) {
    file_usage_add($file, 'MODULE', 'user', $user->uid);
  }
}
```

- Enjoy.

**Note**: likely, this module will not be understandable for newbies. Use it if you'd really know what to do.
